#include <stdio.h>

  /* Particije - Osnovni problem:
       Broj rastava zadanog prirodnog broja n u sumu
     nepadajucih (prirodnih) pribrojnika.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).

     Ucitava (prirodni) broj n,
     ispisuje broj opisanih particija p(n).
     Ispisuje i broj rekurzivnih poziva funkcije.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je prvi <= suma. */

int particije(int suma, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
             Obrada te particije ide na KRAJU, IZA rekurzije,
           zato da generiranje ide u leksikografskom poretku,
           jer ta particija ima najveci pribrojnik i. */

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz osnovno ogranicenje prvi <= i < suma (i = suma se
           obradjuje posebno, bez rekurzije):
           - preostala suma je suma - i,
           - sljedeci prvi je barem i.
           Iz uvjeta da je sljedeci prvi <= preostala suma
           dobivamo i <= suma - i.
           Pripadni test na gornjoj granici petlje po i je
           2*i <= suma, ili i <= max_i = suma / 2. */

    max_i = suma / 2;
    for (i = prvi; i <= max_i; ++i)
        broj += particije(suma - i, i);

        /* Obrada particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p(n).
       Zove rekurzivnu funkciju particije,
       s dodatnim argumentom prvi = 1. */

int p(int n)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           Ako ocekujemo da je n >= 1, onda je 1 = prvi <= suma = n,
           pa NE treba posebna provjera.
           Ako je dozvoljeno n <= 0, onda provjeravamo taj uvjet
           i odmah vratimo p(n), za n <= 0. Dogovorno je p(0) = 1,
           a za n < 0 je p(n) = 0. */
    if (n <= 0) return (n == 0);

        /* Poziv rekurzivne funkcije particije, s prvi = 1. */
    return particije(n, 1);
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Broj particija p(%d) = %d\n",
           n, p(n));
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

    return 0;
}
