#include <stdio.h>
#include <stdlib.h>
#include <math.h>

  /* Sve particije - Varijacija 1:
       Svi rastavi zadanog prirodnog broja n u sumu
     strogo rastucih (prirodnih) pribrojnika.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (pise, broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).

     Ucitava (prirodni) broj n,
     ispisuje sve opisane particije broja n.
     Na kraju, ispisuje broj opisanih particija p_1(n)
     i broj rekurzivnih poziva funkcije.

     Koristi globalno dinamicki alocirano polje pribr za
     pribrojnike u particiji i globalnu varijablu ind_pribr
     za indeks (trenutno) zadnjeg postavljenog pribrojnika u polju.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Globalna varijabla za indeks zadnjeg postavljenog pribrojnika
       (zato da NE mora biti argument rekurzivne funkcije). */

int ind_pribr;


    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Funkcija za ispis jedne (zadnje) nadjene particije.
       Koristi globalno polje pribr i globalnu varijablu
       ind_pribr za indeks zadnjeg pribrojnika u polju.
         Inicijalizacija prije vanjskog poziva rekurzivne
       funkcije je ind_pribr = -1, kao signal da jos nema
       postavljenih pribrojnika. */

void ispis_particije(void)
{
    int i;

        /* Provjera indeksa zadnjeg pribrojnika (za svaki slucaj). */
    if (ind_pribr < 0) return;

        /* Ispis particije, sigurno je ind_pribr >= 0. */
    printf("  %2d", pribr[0]);
    for (i = 1; i <= ind_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");
}

    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je prvi <= suma. */

    /* U trenutku poziva funkcije (ulaza u funkciju), globalni
       ind_pribr = indeks zadnjeg (do sada) postavljenog pribrojnika
       u "globalnoj" particiji polaznog broja. Sljedeci pribrojnik
       se postavlja unutar ovog poziva funkcije. */

int particije_1(int suma, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
             Obrada te particije ide na KRAJU, IZA rekurzije,
           zato da generiranje ide u leksikografskom poretku,
           jer ta particija ima najveci pribrojnik i. */

        /* Povecaj globalni ind_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++ind_pribr;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz osnovno ogranicenje prvi <= i < suma (i = suma se
           obradjuje posebno, bez rekurzije):
           - preostala suma je suma - i,
           - sljedeci prvi je barem i + 1.
           Iz uvjeta da je sljedeci prvi <= preostala suma
           dobivamo i + 1 <= suma - i.
           Pripadni test na gornjoj granici petlje po i je
           2*i + 1 <= suma, ili i <= max_i = (suma - 1) / 2. */

    max_i = (suma - 1) / 2;
    for (i = prvi; i <= max_i; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[ind_pribr] = i;
        broj += particije_1(suma - i, i + 1);
    }

        /* Obrada particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

        /* Postavljanje sljedeceg pribrojnika (i = suma) u polje,
           Ispis nadjene particije, sigurno je ind_pribr >= 0. */
    pribr[ind_pribr] = suma;
    ispis_particije();

        /* Smanji globalni ind_pribr za povratak. */
    --ind_pribr;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_1(n).
       Zove rekurzivnu funkciju particije_1,
       s dodatnim argumentom prvi = 1. */

int p_1(int n)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           Ako ocekujemo da je n >= 1, onda je 1 = prvi <= suma = n,
           pa NE treba posebna provjera.
           Ako je dozvoljeno n <= 0, onda provjeravamo taj uvjet
           i odmah vratimo p_1(n) = 0, za n <= 0.
           Radi jednostavnosti, dogovorno uzimamo da je p_1(0) = 0. */
    if (n <= 0) return 0;

        /* Inicijalizacija globalnog indeksa zadnjeg (do sada)
           postavljenog pribrojnika na -1 (jos ih nema). */
    ind_pribr = -1;

        /* Poziv rekurzivne funkcije particije_1, s prvi = 1. */
    return particije_1(n, 1);
}

int main(void)
{
    int n, max_pribr, broj;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

        /* Najveci moguci broj pribrojnika max_pribr mozemo naci tako
           da stavljamo najmanje moguce pribrojnike (uz zadane uvjete)
           sve dok mozemo, tj. iz
             1 + 2 + ... + max_pribr <= n,
             1 + 2 + ... + max_pribr + (max_pribr + 1) > n.
           Dovoljno je uzeti prvu nejednadzbu i naci najveci takav broj.
           Dobivamo kvadratnu jednadzbu za max_pribr
             (max_pribr)^2 + max_pribr - 2*n <= 0,
           odakle izlazi
             max_pribr <= ( -1 + sqrt( 1 + 8*n ) ) / 2.
        */
    max_pribr = (int) (0.5 * ( -1 + sqrt( 1.0 + 8.0*n ) ));

        /* Alokacija polja za pribrojnike. */
    if ((pribr = (int*) malloc(max_pribr * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Sve trazene particije broja %d su:\n\n", n);

        /* Poziv funkcije - pise i broji sve particije. */
    broj = p_1(n);

        /* Zavrsni ispis broja particija i poziva. */
    printf("\n");
    printf(" Broj particija p_1(%d) = %d\n",
           n, broj);
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

        /* Dealokacija (oslobadjanje) polja za pribrojnike. */
    free(pribr);

    return 0;
}
