#include <stdio.h>
#include <stdlib.h>
#include <math.h>

  /* Sve particije - Varijacija 2:
       Svi rastavi zadanog prirodnog broja n u sumu
     nepadajucih (prirodnih) pribrojnika, s tim da je
     razlika susjednih pribrojnika barem k, gdje je
     k >= 0 zadani cijeli broj.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (pise, broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).
       Varijabla k je globalna (jer se ne mijenja),
     zato da ne mora biti argument rekurzivne funkcije.

     Ucitava (prirodne) brojeve n i k,
     ispisuje sve opisane particije broja n.
     Na kraju, ispisuje broj opisanih particija p_2(n, k)
     i broj rekurzivnih poziva funkcije.

     Koristi globalno dinamicki alocirano polje pribr za
     pribrojnike u particiji i globalnu varijablu ind_pribr
     za indeks (trenutno) zadnjeg postavljenog pribrojnika u polju.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Globalna varijabla za indeks zadnjeg postavljenog pribrojnika
       (zato da NE mora biti argument rekurzivne funkcije). */

int ind_pribr;


    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;

    /* Globalna varijabla k za razliku susjednih pribrojnika. */

int k;


    /* Funkcija za ispis jedne (zadnje) nadjene particije.
       Koristi globalno polje pribr i globalnu varijablu
       ind_pribr za indeks zadnjeg pribrojnika u polju.
         Inicijalizacija prije vanjskog poziva rekurzivne
       funkcije je ind_pribr = -1, kao signal da jos nema
       postavljenih pribrojnika. */

void ispis_particije(void)
{
    int i;

        /* Provjera indeksa zadnjeg pribrojnika (za svaki slucaj). */
    if (ind_pribr < 0) return;

        /* Ispis particije, sigurno je ind_pribr >= 0. */
    printf("  %2d", pribr[0]);
    for (i = 1; i <= ind_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");
}

    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */
    /* Rekurzivna funkcija koristi globalnu varijablu k. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je prvi <= suma. */

    /* U trenutku poziva funkcije (ulaza u funkciju), globalni
       ind_pribr = indeks zadnjeg (do sada) postavljenog pribrojnika
       u "globalnoj" particiji polaznog broja. Sljedeci pribrojnik
       se postavlja unutar ovog poziva funkcije. */

int particije_2(int suma, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
             Obrada te particije ide na KRAJU, IZA rekurzije,
           zato da generiranje ide u leksikografskom poretku,
           jer ta particija ima najveci pribrojnik i. */

        /* Povecaj globalni ind_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++ind_pribr;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz osnovno ogranicenje prvi <= i < suma (i = suma se
           obradjuje posebno, bez rekurzije):
           - preostala suma je suma - i,
           - sljedeci prvi je barem i + k.
           Iz uvjeta da je sljedeci prvi <= preostala suma
           dobivamo i + k <= suma - i.
           Pripadni test na gornjoj granici petlje po i je
           2*i + k <= suma, ili i <= max_i = (suma - k) / 2. */

    max_i = (suma - k) / 2;
    for (i = prvi; i <= max_i; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[ind_pribr] = i;
        broj += particije_2(suma - i, i + k);
    }

        /* Obrada particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

        /* Postavljanje sljedeceg pribrojnika (i = suma) u polje,
           Ispis nadjene particije, sigurno je ind_pribr >= 0. */
    pribr[ind_pribr] = suma;
    ispis_particije();

        /* Smanji globalni ind_pribr za povratak. */
    --ind_pribr;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_2(n, k).
       Zove rekurzivnu funkciju particije_2,
       s dodatnim argumentom prvi = 1. */

int p_2(int n, int kk)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           Ako ocekujemo da je n >= 1 i kk >= 0, onda je
           1 = prvi <= suma = n, pa NE treba posebna provjera.
           Ako je dozvoljeno n <= 0 ili kk < 0, onda to provjeravamo
           i odmah vratimo p_2(n, kk) = 0.
           Radi jednostavnosti, uzimamo da je p_2(0, 0) = 0,
           iako je p(0) = 1. */
    if (n <= 0 || kk < 0) return 0;

        /* Postavi globalni k (za rekurziju) na zadani kk. */
    k = kk;

        /* Inicijalizacija globalnog indeksa zadnjeg (do sada)
           postavljenog pribrojnika na -1 (jos ih nema). */
    ind_pribr = -1;

        /* Poziv rekurzivne funkcije particije_2, s prvi = 1. */
    return particije_2(n, 1);
}

int main(void)
{
    int n, max_pribr, broj;

    printf(" Upisi prirodni broj n i cijeli broj k >= 0: ");
    scanf("%d%d", &n, &k);

        /* Najveci moguci broj pribrojnika max_pribr mozemo naci tako
           da stavljamo najmanje moguce pribrojnike (uz zadane uvjete)
           sve dok mozemo, tj. iz
             1 + (1+k) + (1+2*k) + ... + (1 + (max_pribr - 1)*k) <= n,
             1 + (1+k) + (1+2*k) + ... + (1 + max_pribr*k) > n.
           Dovoljno je uzeti prvu nejednadzbu i naci najveci takav broj.
           Ako je k > 0, dobivamo kvadratnu jednadzbu za max_pribr
             k*(max_pribr)^2 - (k - 2)*max_pribr - 2*n <= 0,
           odakle izlazi
             max_pribr <= ( k - 2 + sqrt( (k - 2)^2 + 8*k*n ) ) / 2.
           Ako je k = 0, onda odmah izlazi max_br <= n.
        */
    if (k > 0)
        max_pribr = (int) (0.5 * ( k - 2
                            + sqrt( (k - 2.0)*(k - 2.0) + 8.0*k*n ) ));
    else
        max_pribr = n;

        /* Alokacija polja za pribrojnike. */
    if ((pribr = (int*) malloc(max_pribr * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Sve trazene particije broja %d su:\n\n", n);

        /* Poziv funkcije - pise i broji sve particije. */
    broj = p_2(n, k);

        /* Zavrsni ispis broja particija i poziva. */
    printf("\n");
    printf(" Broj particija p_2(%d, %d) = %d\n",
           n, k, broj);
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

        /* Dealokacija (oslobadjanje) polja za pribrojnike. */
    free(pribr);

    return 0;
}
