#include <stdio.h>

  /* Particije - Varijacija 3:
       Broj rastava zadanog prirodnog broja n u sumu
     od tocno k nepadajucih (prirodnih) pribrojnika,
     gdje je k zadani prirodan broj.

     Rekurzivno generira (broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).

     Ucitava (prirodne) brojeve n i k,
     ispisuje broj opisanih particija p_3(n, k).
     Ispisuje i broj rekurzivnih poziva funkcije.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je k*prvi <= suma. */

int particije_3(int suma, int k, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je k*prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona s k - 1
           pribrojnika jednaka prvi, a zadnji je preostala suma
           (u ovom trenutku, to je suma - (k - 1)* prvi).
             KRAJ rekurzije je za k = 1, kad treba dodati samo jedan
           preostali pribrojnik. Onda OCEKUJEMO da je prvi <= suma.
           Tada ima tocno JEDNA takva particija - ona s pribrojnikom
           i = suma.
             Obrada te particije ide unutar provjere KRAJA rekurzije,
           BEZ dodatnih rekurzivnih poziva. */

        /* Provjera kraja rekurzije = samo jedan pribrojnik. */
    if (k == 1) {

            /* Obrada particije za i = suma.
               Povecaj broj za 1. */
        ++broj;
    }
    else {    /* Ovdje je k > 1 => nastavak rekurzije. */

            /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
               uz osnovno ogranicenje prvi <= i <= suma:
               - preostala suma je suma - i,
               - broj pribrojnika u preostaloj sumi je jednak k - 1,
               - sljedeci prvi je barem i.
               Iz uvjeta da je (k - 1)*(sljedeci prvi) <= preostala suma
               dobivamo (k - 1)*i <= suma - i.
               Pripadni test na gornjoj granici petlje po i je
               k*i <= suma, ili i <= max_i = suma / k. */

        max_i = suma / k;
        for (i = prvi; i <= max_i; ++i)
            broj += particije_3(suma - i, k - 1, i);
    }

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_3(n, k).
       Zove rekurzivnu funkciju particije_3,
       s dodatnim argumentom prvi = 1. */

int p_3(int n, int k)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           U vanjskom pozivu je prvi = 1, pa mora biti k <= n.
           Ako ocekujemo da je n, k >= 1, onda je to dovoljno.
           Ako je dozvoljeno n <= 0 ili k <= 0, onda osiguravamo
           i da je k >= 1 (uz n >= k, to osigurava i da je n >= 1).
           Drugim rijecima, za k <= 0 ili k > n (pokriva n <= 0),
           odmah vratimo p_3(n, k) = 0. */
    if (k <= 0 || k > n) return 0;

        /* Poziv rekurzivne funkcije particije_3, s prvi = 1. */
    return particije_3(n, k, 1);
}

int main(void)
{
    int n, k;

    printf(" Upisi prirodne brojeve n i k: ");
    scanf("%d%d", &n, &k);

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Broj particija p_3(%d, %d) = %d\n",
           n, k, p_3(n, k));
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

    return 0;
}
