#include <stdio.h>
#include <stdlib.h>

  /* Ucitava n i pise sve particije broja n,
     tj. sve rastave broja n u sumu nepadajucih
     prirodnih brojeva.
     Ispisuje i cijeli "trag" poziva sa svim postavljenim
     pribrojnicima i nadjenim particijama.
     Na kraju ispisuje i broj particija.

     Rekurzivno generira, broji i zbraja sve particije.
     Osnovni algoritam - ima puno rekurzivnih poziva
     koji ne generiraju nove particije,
     tj. ima puno vise poziva od broja particija.
     (v. par_0.c).

     Koristi globalno dinamicki alocirano polje
     za pribrojnike u particiji.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Nadji sve particije broja suma u kojima je prvi (sljedeci)
       pribrojnik barem jednak prvi. */

    /* Argument br_pribr sluzi za ispis = broj do sada postavljenih
       pribrojnika u "globalnoj" particiji polaznog broja, u trenutku
       poziva funkcije (tj., na ulazu u funkciju), a sljedeci
       pribrojnik se postavlja unutar ovog poziva funkcije.
       Sigurno je br_pribr >= 0 (u vanjskom pozivu je br_pribr = 0).
         Za br_pribr > 0, indeks zadnjeg postavljenog pribrojnika u
       polju pribr je br_pribr - 1. */

int particije(int suma, int prvi, int br_pribr)
{
    int i, broj = 0;

        /* Ispis traga poziva. */
    printf("  suma[%2d] = %2d,  prvi[%2d] = %2d",
           br_pribr + 1, suma, br_pribr + 1, prvi);

        /* Ispis svih do sada postavljenih pribrojnika. */
    if (br_pribr > 0) {

            /* Provjera je li particija zavrsena. */
        if (suma == 0)
            printf("   ->");    /* Oznaka za nadjenu particiju. */
        else
            printf("     ");

            /* Ispis particije, sigurno je br_pribr > 0. */
        printf("  %2d", pribr[0]);
        for (i = 1; i < br_pribr; ++i)
            printf(" + %2d", pribr[i]);
        printf("\n");
    }
    else
        printf("       (vanjski poziv - nema pribrojnika)\n");

        /* Povratak za zavrsenu particiju. */
    if (suma == 0) return 1;

        /* else */

        /* Povecaj br_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++br_pribr;

    for (i = prvi; i <= suma; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[br_pribr - 1] = i;
        broj += particije(suma - i, i, br_pribr);
    }
    return broj;
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

        /* Alokacija polja za pribrojnike. */

    if ((pribr = (int*) malloc(n * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    printf("\n Trag poziva funkcije i postavljenih pribrojnika");
    printf(" za particije broja %d:\n\n", n);
    printf("\n Broj particija p(%d) = %d\n",
           n, particije(n, 1, 0) );

    free(pribr);

    return 0;
}
