#include <stdio.h>
#include <stdlib.h>

  /* Ucitava n i pise sve particije broja n,
     tj. sve rastave broja n u sumu nepadajucih
     prirodnih brojeva.
     Na kraju ispisuje i broj particija.

     Rekurzivno generira, broji i zbraja sve particije.
     Efikasni algoritam, BEZ nepotrebnih rekurzivnih poziva,
     tj. broj poziva = broj particija.
     (v. par_4.c).

     Koristi globalno dinamicki alocirano polje
     za pribrojnike u particiji.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Nadji sve particije broja suma u kojima je prvi (sljedeci)
       pribrojnik barem jednak prvi. */
    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne particije.
       Ovdje ocekujemo da je prvi <= suma. */

    /* Argument br_pribr sluzi za ispis = broj do sada postavljenih
       pribrojnika u "globalnoj" particiji polaznog broja, u trenutku
       poziva funkcije (tj., na ulazu u funkciju), a sljedeci
       pribrojnik se postavlja unutar ovog poziva funkcije.
       Sigurno je br_pribr >= 0 (u vanjskom pozivu je br_pribr = 0).
         Za br_pribr > 0, indeks zadnjeg postavljenog pribrojnika u
       polju pribr je br_pribr - 1. */

int particije(int suma, int prvi, int br_pribr)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Povecaj br_pribr za sljedeci pribrojnik i. */
    ++br_pribr;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma. */
        /* Obrada te particije ide na kraju, iza rekurzije. */

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           za i <= suma - i < suma.
           Test na GORNJOJ granici i <= suma - i osigurava da u
           nastavku rekurzije vrijedi prvi <= suma, tj. da postoje
           takve particije u kojima je sljedeci pribrojnik barem i. */

    for (i = prvi; i <= suma - i; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[br_pribr - 1] = i;
        broj += particije(suma - i, i, br_pribr);
    }

        /* Obrada (brojanje) particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

        /* Ispis te particije, sigurno je br_pribr > 0. */
    pribr[br_pribr - 1] = suma;
    printf("  %2d", pribr[0]);
    for (i = 1; i < br_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");

    return broj;
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

        /* Alokacija polja za pribrojnike. */

    if ((pribr = (int*) malloc(n * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

        /* Ovdje je n >= 1, tj. suma >= prvi, pa NE treba
           provjera argumenata za vanjski (prvi) poziv. */

    printf("\n Sve particije broja %d su:\n\n", n);
    printf("\n Broj particija p(%d) = %d\n",
           n, particije(n, 1, 0) );

    printf("\n Broj poziva funkcije = %u\n",
           broj_poziva);

    free(pribr);

    return 0;
}
