#include <stdio.h>
#include <string.h>
#include <stdlib.h>

  /* Sortiranje rjecnika. */

  /* Varijacija 2: rijeci se vise NE pamte u jednom polju w,
     vec se prostor za svaku rijec posebno dinamicki alocira,
     a polje p se dinamicki (re)alocira, kao u varijaciji 1.

     Sortiranje zamjenama pokazivaca u p - izbor ekstrema.

     Funkcija unos vraca status ucitavanja rijeci,
     a broj ucitanih rijeci je globalna varijabla.
     Koristi se kod oslobadjanja memorije.

     Za ucitavanje koristimo da je poznata maksimalna
     moguca duljina rijeci MAXDULJ = 80.
     To je dovoljno za sve prakticne potrebe.
     Nema smisla raditi dinamicko alociranje "znak-po-znak" za
     citanje stringa nepoznate duljine (v. vjezbe, poglavlje 5.2). */

    /* Funkcija my_gets za citanje stringa. */
#include "my_gets.c"


    /* Simbolicka konstanta:
       MAXDULJ = max. duljina pojedine rijeci,
       ukjucivo i nul-znak na kraju stringa. */

#define MAXDULJ 80

    /* Globalno dinamicko polje pokazivaca
       na pojedine rijeci - stringove. */

char **p = NULL;

    /* Globalni stvarni broj rijeci u rjecniku = duljina polja p. */

int broj_rijeci = 0;


    /* Ucitava rijeci i vraca status ucitavanja (0 = nema greske).
       Kraj ucitavanja je prazna rijec.
       Broj ucitanih rijeci broj_rijeci je globalna varijabla. */

int unos(void)
{
    char q[MAXDULJ] = "";    /* Buffer za citanje stringa. */
    char **temp_p;
    int status = 0, broj = 0, dulj;

    while (1) {

            /* Citanje stringa u buffer i test greske. */
        if (my_gets(q, MAXDULJ) == NULL) {
            status = -1;
            break;
        }

        dulj = strlen(q);

            /* Test na praznu rijec - kraj ucitavanja. */
        if (dulj == 0) break;

            /* Imamo nepraznu novu rijec. */

            /* Realociraj p. */
            /* Ako realokacija nije uspjela, vrati do tada
               ucitani p i broj rijeci. */
        temp_p = (char **) realloc(p, (broj + 1) * sizeof(char *));
        if (temp_p == NULL) {
            status = 1;
            break;
        }
            /* Realokacija je uspjela, postavi (mozda) novi p. */
        p = temp_p;

            /* Alociraj prostor za novu rijec. */
            /* Ako alokacija nije uspjela, vrati do tada
               ucitani p i broj rijeci. */
        p[broj] = (char *) malloc((dulj + 1) * sizeof(char));
        if (p[broj] == NULL) {
            status = 2;
            break;
        }

            /* Alokacija je uspjela, kopiraj novu rijec na to mjesto. */
        strcpy(p[broj], q);

            /* Kontrolni ispis. */
        printf(" rijec[%d] = %s\n", broj, p[broj]);

            /* Na kraju, povecaj broj rijeci. */
        ++broj;
    }

        /* Postavi globalni broj_rijeci. */
    broj_rijeci = broj;
    return status;
}

    /* Sortiranje rjecnika izborom ekstrema.
       Dovodimo najmanji element na pocetak.

       Rjecnik je zadan poljem pokazivaca na rijeci
       (element polja je pokazivac na znak-string).
       Koristimo samo zamjene pokazivaca. */

void sort(char *p[], int n)
{
    int i, j, ind_min;
    char *temp;

    for (i = 0; i < n - 1; ++i) {
        ind_min = i;
        for (j = i + 1; j < n; ++j)
            if (strcmp(p[j], p[ind_min]) < 0)
                ind_min = j;
        if (i != ind_min) {
            temp = p[i];
            p[i] = p[ind_min];
            p[ind_min] = temp;
        }
    }

    return;
}

    /* Ispisuje sve rijeci u rjecniku. */

void ispis(char *p[])
{
    int i;

    for (i = 0; i < broj_rijeci; ++i)
        puts(p[i]);

    return;
}

    /* Glavni program. */

int main(void)
{
    int status, i;

    status = unos();
    if (status == -1)
        printf("Greska: EOF u citanju rijeci.\n");
    else if (status == 1)
        printf("Greska: Realokacija za p nije uspjela.\n");
    else if (status == 2)
        printf("Greska: Alokacija za rijec nije uspjela.\n");

        /* Namjerno, u gornjem if, na kraju nema else:
           bez obzira na eventualnu gresku, sortiramo
           sve uspjesno ucitane rijeci. */

    printf("Broj rijeci = %d\n", broj_rijeci);
    printf("\nPolazni rjecnik:\n");
    ispis(p);

        /* Sortiranje polja pokazivaca p. */
    sort(p, broj_rijeci);

    printf("\nSortirani rjecnik:\n");
    ispis(p);

        /* Kod oslobadjanja memorije za pojedine rijeci, koristimo da
           broj_rijeci uvijek sadrzi broj uspjesno alociranih rijeci. */
    for (i = 0; i < broj_rijeci; ++i)
        free(p[i]);
    free(p);

    return 0;
}
