#include <stdio.h>

  /* Primjer za operacije s matricama i vektorima
     uz koristenje tipova. Tipovi sadrze maksimalne
     dimenzije, a prave dimenzije su argumenti funkcija. */

#define MAX_DIM 100

    /* Definicije tipova (imena na engleskom). */

typedef double scalar;
typedef scalar vector[MAX_DIM];
typedef vector matrix[MAX_DIM];

    /* Funkcija za skalarni produkt (dot product)
       dva vektora duljine n, alfa = <x, y>. */

scalar dot_prod(int n, vector x, vector y)
{
    int i;
    scalar prod = 0.0;

    for (i = 0; i < n; ++i)
        prod += x[i] * y[i];
    return prod;
}

    /* Funkcija za zbroj dva vektora duljine n, z = x + y. */

void vec_add(int n, vector x, vector y, vector z)
{
    int i;

    for (i = 0; i < n; ++i)
        z[i] = x[i] + y[i];
    return;
}

    /* Funkcija za zbroj dvije matrice tipa (m, n), C = A + B. */

void mat_add(int m, int n, matrix A, matrix B, matrix C)
{
    int i, j;

    for (i = 0; i < m; ++i)
        for (j = 0; j < n; ++j)
            C[i][j] = A[i][j] + B[i][j];
    return;
}

    /* Funkcija za produkt matrice tipa (m, n) i vektora duljine n,
       y = A * x (rezultat je duljine m). */

void mat_vec_prod(int m, int n, matrix A, vector x, vector y)
{
    int i, j;

    for (i = 0; i < m; ++i) {
        y[i] = 0.0;
        for (j = 0; j < n; ++j)
            y[i] += A[i][j] * x[j];
        }
    return;
}

    /* Funkcija za ispis vektora zadanog imena, duljine n. */

void print_vec(char *ime, int n, vector x)
{
    int i;

    printf("  Vektor %s (transponiran):\n", ime);

        /* Ispis elemenata vektora x. */
    printf("    ");
    for (i = 0; i < n; ++i)
        printf(" %6.2f", x[i]);
    printf("\n");
    return;
}

    /* Funkcija za ispis matrice zadanog imena, tipa (m, n). */

void print_mat(char *ime, int m, int n, matrix A)
{
    int i, j;

    printf("  Matrica %s:\n", ime);

        /* Ispis elemenata matrice A. */
    for (i = 0; i < m; ++i) {
        printf("    %d. red:", i + 1);
        for (j = 0; j < n; ++j)
            printf(" %6.2f", A[i][j]);
        printf("\n");
    }
    return;
}

    /* Glavni program za demo. */

int main(void)
{
        /* Stvarne dimenzije matrica, odnosno, vektora. */
    int m, n;
    vector x, y, z;
    matrix A, B, C;

        /* Stvarna dimenzija vektora je n = 3. */
    n = 3;
    x[0] = 1.0;  y[0] = -1.0;
    x[1] = 2.0;  y[1] =  1.0;
    x[2] = 3.0;  y[2] =  0.0;

    printf(" Pocetni vektori:\n");
    print_vec("x", n, x);
    print_vec("y", n, y);

        /* Skalarni produkt x, y. */
    printf("\n");
    printf(" Skalarni produkt = %g\n", dot_prod(n, x, y));

        /* Zbroj vektora x, y. */
    vec_add(n, x, y, z);
    printf("\n");
    printf(" Zbroj vektora:\n");
    print_vec("z", n, z);

        /* Stvarna dimenzija matrica je (m, n) = (2, 3). */
    m = 2;  n = 3;
    A[0][0] = 2.0;  A[0][1] = 1.0;  A[0][2] = 1.0;
    A[1][0] = 0.0;  A[1][1] = 1.0;  A[1][2] = 0.0;
    B[0][0] = 1.0;  B[0][1] = 0.0;  B[0][2] = 1.0;
    B[1][0] = 2.0;  B[1][1] = 2.0;  B[1][2] = 0.0;

    printf("\n\n");
    printf(" Pocetne matrice:\n");
    print_mat("A", m, n, A);
    print_mat("B", m, n, B);

        /* Zbroj matrica A i B. */ 
    mat_add(m, n, A, B, C);
    printf("\n");
    printf(" Zbroj matrica:\n");
    print_mat("C", m, n, C);

        /* Produkt matrice A i vektora x.
           Rezultat je vektor duljine m = 2. */
    mat_vec_prod(m, n, A, x, y);
    printf("\n");
    printf(" Produkt matrice A i vektora x:\n");
    print_vec("y", m, y);

    return 0;
}
