#include <stdio.h>
#include <stdlib.h>

  /* Vezane liste - Primjer 8.
     Sortirano spajanje (merge) dvije sortirane liste.
     Okretanje (invertiranje) liste.
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

lista merge(lista prvi_1, lista prvi_2)
{
        /* Sortirano spaja dvije liste (merge). */

    lista prvi = NULL, zadnji, pom;

        /* Ako je jedna lista prazna, rezultat je ona druga lista. */
    if (prvi_1 == NULL) return prvi_2;
    if (prvi_2 == NULL) return prvi_1;

        /* U nastavku obrade, obje liste sigurno NISU prazne. */

        /* Prolaz po obje liste, sve dok su obje neprazne. */
    while (prvi_1 != NULL && prvi_2 != NULL) {

            /* Nadji manjeg, od prvog iz prve liste i prvog iz
               druge liste. Izbaci ga s pocetka njegove liste,
               tj. skrati pripadnu listu. */
        if (prvi_1->broj <= prvi_2->broj) {
            pom = prvi_1;
            prvi_1 = prvi_1->sljed;
        }
        else {
            pom = prvi_2;
            prvi_2 = prvi_2->sljed;
        }

            /* Ubaci ga na kraj sortirane liste. */
        if (prvi == NULL)
            prvi = pom;
        else
            zadnji->sljed = pom;
        zadnji = pom;
    }

        /* Spoji neprazni ostatak (tocno jedna lista, druga je prazna)
           na kraj sortirane liste. */
    if (prvi_1 == NULL)
        zadnji->sljed = prvi_2;

    if (prvi_2 == NULL)
        zadnji->sljed = prvi_1;

    return prvi;
}

/* ======================================================================== */

lista okreni_listu(lista prvi)
{
        /* Invertira poredak elemenata u listi. */
        /* Kraca varijanta: prvi = pocetak preostale neokrenute liste. */

    lista preth, ovaj = NULL;

    while (prvi != NULL) {
        preth = ovaj;
        ovaj = prvi;
        prvi = prvi->sljed;
        ovaj->sljed = preth;
    }

    return ovaj;
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi_1 = NULL;    /* Prazna lista! */
    lista prvi_2 = NULL;    /* Prazna lista! */
    lista zadnji_1, zadnji_2;
    lista prvi;
    int broj;

        /* Punjenje prve liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi_1 = kreiraj_straga(prvi_1, &zadnji_1, broj);
    } while (broj != 0);

        /* Punjenje druge liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("\n");
    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi_2 = kreiraj_straga(prvi_2, &zadnji_2, broj);
    } while (broj != 0);

        /* Prva lista:
           ispis broja elemenata i cijele liste. */

    printf("\n Prva lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi_1));
    ispisi_listu(prvi_1);

        /* Druga lista:
           ispis broja elemenata i cijele liste. */

    printf("\n Druga lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi_2));
    ispisi_listu(prvi_2);


        /* Sortirani spoj (merge) dvije liste. */

    prvi = merge(prvi_1, prvi_2);

        /* Spojena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n Spojena lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Invertiranje liste. */

    prvi = okreni_listu(prvi);

        /* Invertirana lista:
           ispis broja elemenata i cijele liste. */

    printf("\n Invertirana lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
