#include <stdio.h>
#include <stdlib.h>

  /* Preuredjenje vezane liste - prvo parni, onda neparni.
     CUVA relativni poredak parnih, odnosno, neparnih elemenata.

     Algoritam 0 (pocetno rjesenje):
     - Rastav polazne liste u dvije zasebne liste, spajanje na kraju.
     - Za osnovne operacije na listama (dodavanje na kraj, spajanje)
       zovemo poznate funkcije (=> sve liste su stalno uzmeljene).
     NIJE efikasan, zbog stalnog uzemljenja (azuriranje pokazivaca)!
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

    /* Funkcije za osnovne operacije s listama, koje su potrebne
       za funkciju preuredi_par_nepar_0. */

/* ======================================================================== */

lista ubaci_na_kraj(lista prvi, lista *p_zadnji, lista novi)
{
        /* Ne provjerava novi != NULL. */
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;

        /* Moze: prvi == NULL || zadnji == NULL. */
    if (prvi == NULL)
        prvi = novi;
    else    /* Ocekujemo zadnji->sljed == NULL. */
        zadnji->sljed = novi;
    novi->sljed = NULL;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista spoji_dvije(lista prvi_1, lista prvi_2)
{
    lista pom;

        /* Ako je prva lista prazna, rezultat je druga lista. */
    if (prvi_1 == NULL) return prvi_2;

        /* Nadji zadnjeg u prvoj i spoji drugu listu na njega. */
    for (pom = prvi_1; pom->sljed != NULL; pom = pom->sljed);
    pom->sljed = prvi_2;

    return prvi_1;
}

/* ======================================================================== */

    /* Preuredjuje listu tako da na pocetku liste budu svi parni,
       a zatim dolaze svi neparni elementi iz polazne liste.
       Relativni poredak parnih, odnosno, neparnih elemenata
       je ISTI kao u polaznoj listi. */

    /* Algoritam 0:
         Rastavi polaznu listu u dvije zasebne liste (parni, neparni),
       a zatim ih spoji (konkatencija).

         Za CUVANJE relativnog poretka, radimo ubacivanje na KRAJ
       odgovarajuce liste. Zato pamtimo prvi i zadnji element u
       obje liste (parni, neparni). Tu koristimo poznatu funkciju
       ubaci_na_kraj za ubacivanje na kraj (s pamcenjem zadnjeg).
       Kod spajanja, koristimo poznatu funkciju spoji_dvije.
    */

lista preuredi_par_nepar_0(lista prvi)
{
        /* Inicijalizacije liste parnih i liste neparnih na praznu listu.
           Za obje liste pamtimo pokazivace na prvi i zadnji element.
           Pokazivace na prvi element inicijaliziramo na NULL.
           Pokazivace na zadnji element NE treba inicijalizirati na NULL. */
    lista prvi_parni = NULL, zadnji_parni;
    lista prvi_neparni = NULL, zadnji_neparni;

        /* Pokazivac na radni element (onaj kojeg obradjujemo). */
    lista radni;

        /* Petlja za prolaz po polaznoj listi.
           Napreduje za po jedan radni element. */

    while (prvi != NULL) {

            /* Izbaci prvi preostali element (= radni) i makni pocetak. */
        radni = prvi;
        prvi = prvi->sljed;

            /* Sad obradjujemo izbaceni radni element, tj. ubacujemo
               ga na KRAJ odgovarajuce liste, ovisno o parnosti broja. */

        if (radni->broj % 2 == 0) {    /* Parni. */

                /* Ubaci ga na kraj liste parnih. */
            prvi_parni = ubaci_na_kraj(prvi_parni,
                                       &zadnji_parni, radni);
        }
        else {    /* Neparni. */

                /* Ubaci ga na kraj liste neparnih. */
            prvi_neparni = ubaci_na_kraj(prvi_neparni,
                                         &zadnji_neparni, radni);
        }
    }

        /* Spajanje (konkatenacija) liste parnih i liste neparnih.
           Odmah vracamo pokazivac na pocetak spojene liste. */

    return spoji_dvije(prvi_parni, prvi_neparni);
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Preuredjenje liste - prvo parni, onda neparni. */

    prvi = preuredi_par_nepar_0(prvi);

        /* Preuredjena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Preuredjena lista (parni, neparni):\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
