/* Zadatak 1A:

*/

#include <stdio.h>
#include <stdlib.h>

    /* Globalna deklaracija tipa. */

typedef struct {
    char ime[30];
    char prezime[30];
    int broj;
    char ekipa[3];
    double rezultat[21];
} biciklist;

    /* Pomocna funkcija.
       Preskace niz od nula ili vise praznina u datoteci ulaz
       i vraca prvi znak koji nije praznina (blank). */

int preskoci_praznine(FILE* ulaz)
{
    int ch;
    while ((ch = fgetc(ulaz)) == ' ');
    return ch;
}

    /* Funkcija za citanje jednog biciklista. */

biciklist* ucitaj(FILE* ulaz)
{
    biciklist* p_bic;
    int sep = '|';
    int ch, m, i;
    int h, mm, ss, stotinke;

    p_bic = (biciklist*) malloc(sizeof(biciklist));

        /* 1. red. */
    fscanf(ulaz, "%[^,],%*c%[^\n]%*c", p_bic->ime, p_bic->prezime);

        /* 2. red. */
/* Kraticu mozemo ucitati petljom (znak po znak), sljedecim
   nizom naredbi:

    for (i = 0; i < 3; ++i)
        fscanf(ulaz, "%c", &(p_bic->ekipa[i]));

   Umjesto toga, cijelu kraticu citamo odjednom - kao polje s 3 znaka:
*/
    fscanf(ulaz, "%3c", p_bic->ekipa);
    fgetc(ulaz);
    fscanf(ulaz, "%d\n", &(p_bic->broj));

        /* 3. red. */
    m = 0;
        /* Petlja, sve dok ne ucitamo '\n' (kraj reda). */
    do {
        ch = preskoci_praznine(ulaz);

            /* Ako je znak ch == sep ili '\n' (kraj reda),
               pripadni blok je prazan (nema vremena). */
        if (ch == sep || ch == '\n') {
            p_bic->rezultat[m] = -1.0;
        }
            /* U protivnom, blok nije prazan (ima vrijeme),
               znak ch mora biti znamenka i to su sati! */
        else {
/* 1. mogucnost:
   Znak ch treba pretvoriti u znamenku jedinica za sate (znak -> broj).
   Ucitamo nastavak vremena (bez prvog znaka za sate) u zadanom formatu,
   pocevsi od znaka ':'.
   Onda sve pretvorimo u sekunde, s tim da znamenku jedinica za sate
   treba korektno dodati. To se moze napraviti sljedecim naredbama:

            fscanf(ulaz, ":%d:%d,%d", &mm, &ss, &stotinke);
            h = ch - '0';
            p_bic->rezultat[m] = h * 3600.0 + mm * 60.0 + ss
                                   + stotinke / 100.0;

   2. mogucnost:
   Vratimo znak ch NATRAG u spremnik za ponovno citanje (ungetc),
   ucitamo CIJELO vrijeme u zadanom formatu i sve pretvorimo u sekunde:
*/
            ungetc(ch, ulaz);
            fscanf(ulaz, "%d:%d:%d,%d", &h, &mm, &ss, &stotinke);
            p_bic->rezultat[m] = h * 3600.0 + mm * 60.0 + ss
                                   + stotinke / 100.0;

            ch = preskoci_praznine(ulaz);
        }
            /* Povecaj broj obradjenih blokova. */
        ++m;
            /* Ovdje je sigurno ch == sep ili '\n'.
               Provjeri kraj reda = prekid petlje,
               tj. nastavi petlju ako ch nije '\n'. */
    } while (ch != '\n');

        /* Postavi preostala vremena, za koja nije bilo blokova. */
    for (i = m; i < 21; ++i)
        p_bic->rezultat[i] = -1.0;

        /* Vrati pokazivac na strukturu. */
    return p_bic;
}

    /* Glavni program za demo. */

int main(void)
{
    FILE *ulaz;
    char* ime = "1a.txt";
    biciklist* p_bic;
    int k, i;
    int ch;

        /* Otvaranje ulazne tekst-datoteke. */
    if ((ulaz = fopen(ime, "r")) == NULL) {
        fprintf(stderr, "Ne mogu citati: %s!\n", ime);
        exit(1);
    }

        /* Ucitaj sve bicikliste, do kraja datoteke. */
    k = 0;
    while (1) {
        ch = fgetc(ulaz);
        if (ch == EOF) break;

            /* Ako nije EOF, onda smo upravo ucitali prvi znak
               u sljedecem redu (pocetak sljedeceg podatka).
               Umjesto posebne obrade tog znaka, vratimo ga
               NATRAG u spremnik, za ponovno citanje.
               Funkcija ucitaj ce ga jos jednom procitati. */
        ungetc(ch, ulaz);

        p_bic = ucitaj(ulaz);

            /* Ispis pripadnih podataka za kontrolu. */
        printf(" %d. biciklist:\n", k + 1);
        printf("       Ime = \"%s\"\n", p_bic->ime);
        printf("   Prezime = \"%s\"\n", p_bic->prezime);
        printf("      Broj = %d\n", p_bic->broj);
/* Kraticu mozemo napisati petljom (znak po znak), sljedecim
   nizom naredbi:

        printf("     Ekipa = ");
        for (i = 0; i < 3; ++i)
            printf("%c", p_bic->ekipa[i]);
        printf("\n");

   Umjesto toga, cijelu kraticu pisemo odjednom - kao string,
   ali tako da pisemo najvise 3 znaka iz polja:
*/
        printf("     Ekipa = %.3s\n", p_bic->ekipa);
            /* Rezultati. */
        for (i = 0; i < 21; ++i)
            printf("   Rezultat[%2d] = %8.2f\n", i, p_bic->rezultat[i]);
        printf("\n");

        ++k;
        free(p_bic);
    }

    fclose(ulaz);

    return 0;
}
