/* Zadatak 1D:

*/

#include <stdio.h>
#include <stdlib.h>

    /* Globalna deklaracija tipa. */

typedef struct {
    char ime[35];
    char prezime[35];
    int broj;
    char drzava[3];
    double rezultat[24];
} trkac;

    /* Pomocna funkcija.
       Preskace niz od nula ili vise praznina u datoteci ulaz
       i vraca prvi znak koji nije praznina (blank). */

int preskoci_praznine(FILE* ulaz)
{
    int ch;
    while ((ch = fgetc(ulaz)) == ' ');
    return ch;
}

    /* Funkcija za citanje jednog trkaca. */

trkac* ucitaj(FILE* ulaz)
{
    trkac* p_trk;
    int sep = '!';
    int ch, m, i;
    int h, mm, ss, desetinke;

    p_trk = (trkac*) malloc(sizeof(trkac));

        /* 1. red. */
    fscanf(ulaz, "%[^,],%*c%[^\n]%*c", p_trk->ime, p_trk->prezime);

        /* 2. red. */
/* Kraticu mozemo ucitati petljom (znak po znak), sljedecim
   nizom naredbi:

    for (i = 0; i < 3; ++i)
        fscanf(ulaz, "%c", &(p_trk->drzava[i]));

   Umjesto toga, cijelu kraticu citamo odjednom - kao polje s 3 znaka:
*/
    fscanf(ulaz, "%3c", p_trk->drzava);
    fgetc(ulaz);
    fscanf(ulaz, "%d\n", &(p_trk->broj));

        /* 3. red. */
    m = 0;
        /* Petlja, sve dok ne ucitamo '\n' (kraj reda). */
    do {
        ch = preskoci_praznine(ulaz);

            /* Ako je znak ch == sep ili '\n' (kraj reda),
               pripadni blok je prazan (nema vremena). */
        if (ch == sep || ch == '\n') {
            p_trk->rezultat[m] = -1.0;
        }
            /* U protivnom, blok nije prazan (ima vrijeme),
               znak ch mora biti znamenka i to su sati! */
        else {
/* 1. mogucnost:
   Znak ch treba pretvoriti u znamenku jedinica za sate (znak -> broj).
   Ucitamo nastavak vremena (bez prvog znaka za sate) u zadanom formatu,
   pocevsi od znaka ':'.
   Onda sve pretvorimo u sekunde, s tim da znamenku jedinica za sate
   treba korektno dodati. To se moze napraviti sljedecim naredbama:

            fscanf(ulaz, ":%d:%d,%d", &mm, &ss, &desetinke);
            h = ch - '0';
            p_trk->rezultat[m] = h * 3600.0 + mm * 60.0 + ss
                                   + desetinke / 10.0;

   2. mogucnost:
   Vratimo znak ch NATRAG u spremnik za ponovno citanje (ungetc),
   ucitamo CIJELO vrijeme u zadanom formatu i sve pretvorimo u sekunde:
*/
            ungetc(ch, ulaz);
            fscanf(ulaz, "%d:%d:%d,%d", &h, &mm, &ss, &desetinke);
            p_trk->rezultat[m] = h * 3600.0 + mm * 60.0 + ss
                                   + desetinke / 10.0;

            ch = preskoci_praznine(ulaz);
        }
            /* Povecaj broj obradjenih blokova. */
        ++m;
            /* Ovdje je sigurno ch == sep ili '\n'.
               Provjeri kraj reda = prekid petlje,
               tj. nastavi petlju ako ch nije '\n'. */
    } while (ch != '\n');

        /* Postavi preostala vremena, za koja nije bilo blokova. */
    for (i = m; i < 24; ++i)
        p_trk->rezultat[i] = -1.0;

        /* Vrati pokazivac na strukturu. */
    return p_trk;
}

    /* Glavni program za demo. */

int main(void)
{
    FILE *ulaz;
    char* ime = "1d.txt";
    trkac* p_trk;
    int k, i;
    int ch;

        /* Otvaranje ulazne tekst-datoteke. */
    if ((ulaz = fopen(ime, "r")) == NULL) {
        fprintf(stderr, "Ne mogu citati: %s!\n", ime);
        exit(1);
    }

        /* Ucitaj sve trkace, do kraja datoteke. */
    k = 0;
    while (1) {
        ch = fgetc(ulaz);
        if (ch == EOF) break;

            /* Ako nije EOF, onda smo upravo ucitali prvi znak
               u sljedecem redu (pocetak sljedeceg podatka).
               Umjesto posebne obrade tog znaka, vratimo ga
               NATRAG u spremnik, za ponovno citanje.
               Funkcija ucitaj ce ga jos jednom procitati. */
        ungetc(ch, ulaz);

        p_trk = ucitaj(ulaz);

            /* Ispis pripadnih podataka za kontrolu. */
        printf(" %d. trkac:\n", k + 1);
        printf("       Ime = \"%s\"\n", p_trk->ime);
        printf("   Prezime = \"%s\"\n", p_trk->prezime);
        printf("      Broj = %d\n", p_trk->broj);
/* Kraticu mozemo napisati petljom (znak po znak), sljedecim
   nizom naredbi:

        printf("    Drzava = ");
        for (i = 0; i < 3; ++i)
            printf("%c", p_trk->drzava[i]);
        printf("\n");

   Umjesto toga, cijelu kraticu pisemo odjednom - kao string,
   ali tako da pisemo najvise 3 znaka iz polja:
*/
        printf("    Drzava = %.3s\n", p_trk->drzava);
            /* Rezultati. */
        for (i = 0; i < 24; ++i)
            printf("   Rezultat[%2d] = %7.1f\n", i, p_trk->rezultat[i]);
        printf("\n");

        ++k;
        free(p_trk);
    }

    fclose(ulaz);

    return 0;
}
