#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

  /* Program za sortiranje datoteke cijelih brojeva.
     Algoritam = Bubble_sort:
       Sortiranje zamjenama susjeda, uz skracenu vanjsku petlju,
       sve dok ima zamjena u unutarnjoj petlji.
     Predlozak: bubble_2 (Prog_1, 13_PROGS). */

  /* Ubrzana varijanta:
     - susjedi imaju indekse j - 1, j (a ne j, j + 1), cita se x[j],
     - koristi sekvencijalno citanje unaprijed,
     - koristi lokalnost zamjena susjeda, za pomak/pisanje kod zamjena.

     Predlozak: bubble_2_f (ovdje, na S_Polje).
       Umjesto do-while, koristi silaznu for petlju, a izlazak
       iz petlje je s break.
       Indeks j unutarnje petlje je veci za 1.
  */

/* ======================================================================== */

    /* Bubble_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_bubble_sort(FILE *f_obj,
                   long int n_obj, size_t obj_size,
                   int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    obj_type x_jm1, x_j;    /* x_jm1 = x[j - 1], x_j = x[j]. */
    int zamjena, zamjena_j;

        /* Pomak za dva objekta natrag kod zamjene. */
    long int dva_natrag = -2L * obj_size;


        /***** Bubble_sort. *****/

        /* Vanjska petlja za prolaze kroz datoteku.
           Indeks i je indeks zadnjeg elementa do kojeg
           treba ici u sljedecem prolazu kroz datoteku. */
    for (i = n_obj - 1L; i >= 1L; --i) {

        rewind(f_obj);

            /* Ucitaj element x[j], s indeksom j = 0. */
        ucitaj_obj(f_obj, &x_j, obj_size);

        zamjena = 0;
        zamjena_j = 0;

            /* Petlja za citanje sljedeceg elementa.
               Indeks j unutarnje petlje je veci za 1,
               tako da se sljedeci element cita u x_j = x[j]. */
        for (j = 1L; j <= i; ++j) {

                /* Ako u proslom prolazu kroz petlju po j
                   NIJE bila zamjena x[j - 1] i x[j], onda je
                   novi element x[j - 1] = raniji x[j].
                   U protivnom, x_jm1 vec sadrzi prethodni element,
                   kao da smo napravili zamjenu. */
            if (!zamjena_j) x_jm1 = x_j;
            zamjena_j = 0;

                /* Ucitaj element x[j]. */
            ucitaj_obj(f_obj, &x_j, obj_size);

            if (obj_cmp(&x_jm1, &x_j) > 0) {

                    /* Zamjena x[j - 1] i x[j]. */

                    /* Okreni ih u datoteci:
                         makni se dva elementa natrag,
                         prvo napisi x[j], a zatim x[j - 1]. */
                postavi_fpos(f_obj, dva_natrag, SEEK_CUR);
                napisi_obj(f_obj, &x_j, obj_size);
                napisi_obj(f_obj, &x_jm1, obj_size);

                    /* Isprazni spremnik u datoteku nakon pisanja,
                       pri prijelazu s pisanja na citanje. */
                isprazni_spremnik(f_obj);

                zamjena = 1;

                    /* Umjesto zamjene x_jm1 = x[j - 1] i x_j = x[j],
                       ostavljamo stari x[j - 1] (= novi x[j]) u x_jm1,
                       kao prethodni element za sljedeci prolaz.
                       Ako nema zamjene onda treba postaviti novi
                       prethodni element na x_j (test je na pocetku). */
                zamjena_j = 1;
            }
        }

        if (!zamjena) break;
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_bubble_sort(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
