#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

  /* Program za sortiranje datoteke cijelih brojeva.
     Algoritam = Insertion_sort:
       Sortiranje (sortiranim) umetanjem - sortirani dio je spijeda,
       uz sekvencijalno trazenje prema pocetku.
       Izbjegava nepotrebna pisanja ako je novi element najveci.
       Za datoteke, dva kopiranja su nuzna - kod citanja.
     Predlozak: ins_1 (Prog_1, 13_PROGS). */

  /* Standardna varijanta - koristi genericke funkcije na datoteci,
     koje simuliraju odgovarajuce operacije na polju.
  */

/* ======================================================================== */

    /* Insertion_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Genericka varijanta, simulacijom rada na polju. */

void g_insertion_sort(FILE *f_obj,
                   long int n_obj, size_t obj_size,
                   int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    obj_type x_i, x_jm1;    /* x_i = temp = x[i], x_jm1 = x[j - 1]. */


        /***** Insertion_sort. *****/

        /* Vanjska petlja za prolaze kroz datoteku.
           Indeks i je indeks sljedeceg elementa x[i],
           kojeg sortirano ubacujemo u sortirani prednji dio. */
    for (i = 1L; i < n_obj; ++i) {

            /* Ucitaj elemente x[i - 1] u x_jm1 i x[i] u x_i = temp.
               Ova kopiranja su nuzna kod datoteka. */
        ucitaj_obj_i(f_obj, i - 1, &x_jm1, obj_size);
        ucitaj_obj_i(f_obj, i, &x_i, obj_size);

            /* Usporedba x_i = x[i] i x_jm1 = x[i - 1]. */
        if (obj_cmp(&x_i, &x_jm1) < 0) {

                /* Ima pomaka. Ovdje je x_i = temp = x[i].
                   Silazna petlja po j, za pomake (kopiranja). */
/*****
            j = i;
            do {
                x[j] = x[j - 1]; --j;
            } while (j >= 1 && x[j - 1] > temp);
            x[j] = temp;
*****/
            j = i;
            do {
                    /* Kopiraj x_jm1 = x[j - 1] u x[j] (pomak). */
                napisi_obj_i(f_obj, j, &x_jm1, obj_size);

                    /* Smanji j, provjeri je li j >= 1 za nastavak. */
                --j;
                if (j < 1L) break;

                    /* Ucitaj element x[j - 1]. */
                ucitaj_obj_i(f_obj, j - 1, &x_jm1, obj_size);

                    /* Usporedba x_jm1 = x[j - 1] i x_i = temp
                       je ugradjena u uvjet do-while petlje. */
            } while (obj_cmp(&x_jm1, &x_i) > 0);

                /* Umetanje: napisi x_i = temp na mjesto x[j]. */
            napisi_obj_i(f_obj, j, &x_i, obj_size);
        }
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    g_insertion_sort(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
