#include <stdio.h>

  /* Primjer portabilnog generatora pseudoslucajnih brojeva.
     Implementacija funkcija rand, srand.
     U trenutku pisanja, ovo odgovara implementaciji u
     Intel C, Microsoft Visual C, Code::Blocks (17.12). */

    /* Sljedeci pseudoslucajni broj, tipa unsigned long int.
       Deklariran kao static, tako da se cuva vrijednost
       izmedju poziva funkcije rand(). Koristi se i u srand.
       Inicijaliziran na standardno sjeme = 1. */

static unsigned long int next = 1;

    /* Implementacija funkcije rand.
       Pretpostavlja da je RAND_MAX = 32767 = 2^15 - 1.
       Eksplicitno koristi RAND_MAX + 1 = 32768 = 2^15
       kao djelitelj za ostatak, da dobije 15-bitni int.

       Linearni (multiplikativno-aditivni) kongruencijski generator (LCG):
         y_0 = seed;
         y_n = (214013 * y_(n - 1) + 2531011) mod 2^32;
         x_n = bitovi 30..16 iz y_n;    [= 15 bitova]

       Period generatora = 2^32.

       Implementacija: Aritmetika je u tipu unsigned long int
       (ocekuje barem 32 bita), a ostatak se pretvara u int. */

int rand(void)    /* Pretpostavlja RAND_MAX = 32767. */
{
    next = next * 214013 + 2531011;
    return (unsigned int) (next / 65536) % 32768;
}

    /* Implementacija funkcije srand.
       Samo postavlja globalni next na zadani seed. */

void srand(unsigned int seed)
{
    next = seed;
}

    /* Postavlja sjeme za rand() na seed i
       ispisuje prvih n slucajnih brojeva
       iz rand(), s tim sjemenom. */

void ispis_rand(unsigned int seed, int n)
{
    int i;

    srand(seed);

    printf("\n Sjeme (seed) za rand = %u\n", seed);
    printf(" Slucajni brojevi:\n");

    for (i = 1; i <= n; ++i)
        printf(" %6d\n", rand());

    return;
}

    /* Definicija RAND_MAX, samo za ispis. */

#define RAND_MAX 32767

    /* Glavni program za testiranje.
       Ispisuje prvih nekoliko slucajnih brojeva
       s razlicitim sjemenima (zadnje se ucitava). */

int main(void)
{
    unsigned int seed;

    printf("RAND_MAX = %d\n", RAND_MAX);    /* 32767 */

    ispis_rand(0u, 10);
    ispis_rand(1u, 10);
    ispis_rand(2u, 10);
    ispis_rand(3u, 10);
    ispis_rand(4u, 10);
    ispis_rand(5u, 10);

    printf("\n Upisi sjeme: ");
    scanf("%u", &seed);
    ispis_rand(seed, 10);

    return 0;
}
